@@ -48,6 +48,13 @@ typedef struct {
 } DocInfoURI;
 
 struct _YelpDocInfo {
+    gchar      *id;
+    gchar      *title;
+    gchar      *description;
+    gchar      *category;
+    gchar      *lang;
+    gint        lang_priority;
+
     DocInfoURI *uris;
     gint        num_uris;
     gint        max_uris;
@@ -59,19 +66,18 @@ struct _YelpDocInfo {
     gint ref_count;
 };
 
-static void         doc_info_add_uri   (YelpDocInfo *doc_info,
-					gchar       *uri,
-					YelpURIType  type);
+static YelpDocType  get_doc_type       (gchar   *uri);
 static gchar *      convert_ghelp_uri  (gchar   *uri);
+
 static gchar *      convert_man_uri    (gchar   *uri);
 static gchar *      convert_info_uri   (gchar   *uri);
-static YelpDocType  get_doc_type       (gchar   *uri);
 
 YelpDocInfo *
-yelp_doc_info_new (gchar *uri)
+yelp_doc_info_new (const gchar *uri)
 {
     YelpDocInfo *doc;
-    gchar       *doc_uri = NULL;
+    gchar       *doc_uri  = NULL;
+    gchar       *full_uri = NULL;
     YelpDocType  doc_type;
     YelpURIType  uri_type;
     gchar *cur;
@@ -79,39 +85,52 @@ yelp_doc_info_new (gchar *uri)
     g_return_val_if_fail (uri != NULL, NULL);
 
     d (g_print ("yelp_doc_info_new\n"));
-    d (g_print ("  uri     = \"%s\"\n", uri));
+    d (g_print ("  uri      = \"%s\"\n", uri));
 
-    if (g_str_has_prefix (uri, "file:")) {
-	if ((cur = strchr (uri, '#')))
-	    doc_uri = g_strndup (uri, cur - uri);
+    full_uri =
+	gnome_vfs_make_uri_from_input_with_dirs	(uri,
+						 GNOME_VFS_MAKE_URI_DIR_CURRENT);
+
+    if (g_str_has_prefix (full_uri, "file:")) {
+	if ((cur = strchr (full_uri, '#')))
+	    doc_uri = g_strndup (full_uri, cur - full_uri);
 	else
-	    doc_uri = g_strdup (uri);
+	    doc_uri = g_strdup (full_uri);
 	doc_type = get_doc_type (doc_uri);
 	uri_type = YELP_URI_TYPE_FILE;
     }
-    if (g_str_has_prefix (uri, "ghelp:")) {
-	doc_uri  = convert_ghelp_uri (uri);
+    else if (g_str_has_prefix (full_uri, "ghelp:") ||
+	g_str_has_prefix (full_uri, "gnome-help:")) {
+	doc_uri  = convert_ghelp_uri (full_uri);
 	if (doc_uri)
 	    doc_type = get_doc_type (doc_uri);
 	uri_type = YELP_URI_TYPE_GHELP;
     }
-    else if (g_str_has_prefix (uri, "man:")) {
-	doc_uri  = convert_man_uri (uri);
+    else if (g_str_has_prefix (full_uri, "man:")) {
+	doc_uri  = convert_man_uri (full_uri);
 	doc_type = YELP_DOC_TYPE_MAN;
 	uri_type = YELP_URI_TYPE_MAN;
     }
-    else if (g_str_has_prefix (uri, "info:")) {
-	doc_uri  = convert_info_uri (uri);
+    else if (g_str_has_prefix (full_uri, "info:")) {
+	doc_uri  = convert_info_uri (full_uri);
 	doc_type = YELP_DOC_TYPE_INFO;
 	uri_type = YELP_URI_TYPE_INFO;
     }
-    else if (g_str_has_prefix (uri, "x-yelp-toc:")) {
+    else if (g_str_has_prefix (full_uri, "x-yelp-toc:")) {
 	doc_uri = g_strdup ("file://" DATADIR "/yelp/toc.xml");
 	doc_type = YELP_DOC_TYPE_TOC;
 	uri_type = YELP_URI_TYPE_TOC;
     }
+    else {
+	doc_uri = g_strdup (uri);
+	doc_type = YELP_DOC_TYPE_EXTERNAL;
+	uri_type = YELP_URI_TYPE_EXTERNAL;
+    }
 
-    d (g_print ("  doc_uri = \"%s\"\n", doc_uri));
+    d (g_print ("  full_uri = \"%s\"\n", full_uri));
+    d (g_print ("  doc_uri  = \"%s\"\n", doc_uri));
+
+    g_free (full_uri);
 
     if (doc_uri) {
 	doc = g_new0 (YelpDocInfo, 1);
@@ -119,9 +138,9 @@ yelp_doc_info_new (gchar *uri)
 	doc->num_uris = 0;
 	doc->max_uris = 8;
 
-	doc_info_add_uri (doc, doc_uri, YELP_URI_TYPE_FILE);
+	yelp_doc_info_add_uri (doc, doc_uri, YELP_URI_TYPE_FILE);
 	if (uri_type && uri_type != YELP_URI_TYPE_FILE)
-	    doc_info_add_uri (doc, uri, uri_type);
+	    yelp_doc_info_add_uri (doc, uri, uri_type);
 
 	doc->type = doc_type;
 	doc->ref_count = 1;
@@ -132,10 +151,11 @@ yelp_doc_info_new (gchar *uri)
 }
 
 YelpDocInfo *
-yelp_doc_info_get (gchar *uri)
+yelp_doc_info_get (const gchar *uri)
 {
     YelpDocInfo *doc;
     gint i;
+    gchar *c, *doc_uri;
 
     g_return_val_if_fail (uri != NULL, NULL);
 
@@ -149,10 +169,19 @@ yelp_doc_info_get (gchar *uri)
 				   g_free,
 				   (GDestroyNotify) yelp_doc_info_unref);
 
-    doc = (YelpDocInfo *) g_hash_table_lookup (doc_info_table, uri);
+    c = strchr (uri, '?');
+    if (c == NULL)
+	c = strchr (uri, '#');
+
+    if (c != NULL)
+	doc_uri = g_strndup (uri, c - uri);
+    else
+	doc_uri = g_strdup (uri);
+
+    doc = (YelpDocInfo *) g_hash_table_lookup (doc_info_table, doc_uri);
 
     if (!doc) {
-	doc = yelp_doc_info_new (uri);
+	doc = yelp_doc_info_new (doc_uri);
 	if (doc && doc->type != YELP_DOC_TYPE_EXTERNAL) {
 	    YelpDocInfo *old_doc;
 
@@ -165,9 +194,9 @@ yelp_doc_info_get (gchar *uri)
 
 	    if (old_doc) {
 		for (i = 0; i < doc->num_uris; i++) {
-		    doc_info_add_uri (old_doc,
-				      (doc->uris + i)->uri,
-				      (doc->uris + i)->type);
+		    yelp_doc_info_add_uri (old_doc,
+					   (doc->uris + i)->uri,
+					   (doc->uris + i)->type);
 		    g_hash_table_insert (doc_info_table,
 					 g_strdup ((doc->uris + i)->uri),
 					 yelp_doc_info_ref (old_doc));
@@ -184,6 +213,7 @@ yelp_doc_info_get (gchar *uri)
 	}
     }
 
+    g_free (doc_uri);
     return doc;
 }
 
@@ -215,8 +245,10 @@ yelp_doc_info_free (YelpDocInfo *doc)
     if (!doc)
 	return;
 
-    g_object_unref (doc->pager);
+    if (doc->pager)
+	g_object_unref (doc->pager);
 
+    g_free (doc->title);
     for (i = 0; i < doc->num_uris; i++)
 	g_free ((doc->uris + i)->uri);
     g_free (doc->uris);
@@ -245,6 +277,98 @@ yelp_doc_info_set_pager (YelpDocInfo *doc, YelpPager *pager)
     doc->pager = pager;
 }
 
+const gchar *
+yelp_doc_info_get_id (YelpDocInfo *doc)
+{
+    return (const gchar *) doc->id;
+}
+
+void
+yelp_doc_info_set_id (YelpDocInfo *doc, gchar *id)
+{
+    if (doc->id)
+	g_free (doc->id);
+
+    doc->id = g_strdup (id);
+}
+
+const gchar *
+yelp_doc_info_get_title (YelpDocInfo *doc)
+{
+    return (const gchar *) doc->title;
+}
+
+void
+yelp_doc_info_set_title (YelpDocInfo *doc, gchar *title)
+{
+    if (doc->title)
+	g_free (doc->title);
+
+    doc->title = g_strdup (title);
+}
+
+const gchar *
+yelp_doc_info_get_description (YelpDocInfo *doc)
+{
+    return (const gchar *) doc->description;
+}
+
+void
+yelp_doc_info_set_description (YelpDocInfo *doc, gchar *desc)
+{
+    if (doc->description)
+	g_free (doc->description);
+
+    doc->description = g_strdup (desc);
+}
+
+const gchar *
+yelp_doc_info_get_category (YelpDocInfo *doc)
+{
+    return (const gchar *) doc->category;
+}
+
+void
+yelp_doc_info_set_category (YelpDocInfo *doc, gchar *category)
+{
+    if (doc->category)
+	g_free (doc->category);
+
+    doc->category = g_strdup (category);
+}
+
+const gchar *
+yelp_doc_info_get_language (YelpDocInfo *doc)
+{
+    return (const gchar *) doc->lang;
+}
+
+void
+yelp_doc_info_set_language (YelpDocInfo *doc, gchar *language)
+{
+    gint i;
+    const gchar * const * langs = g_get_language_names ();
+
+    if (doc->lang)
+	g_free (doc->lang);
+
+    doc->lang = g_strdup (language);
+
+    doc->lang_priority = INT_MAX;
+    for (i = 0; langs[i] != NULL; i++) {
+	if (g_str_equal (language, langs[i])) {
+	    doc->lang_priority = i;
+	    break;
+	}
+    }
+}
+
+gint
+yelp_doc_info_cmp_language (YelpDocInfo *doc1, YelpDocInfo *doc2)
+{
+    return CLAMP (doc1->lang_priority - doc2->lang_priority, -1, 1);
+}
+
 YelpDocType
 yelp_doc_info_get_type (YelpDocInfo *doc)
 {
@@ -331,7 +455,7 @@ yelp_doc_page_free (YelpDocPage *page)
 }
 
 gchar *
-yelp_uri_get_fragment (gchar *uri)
+yelp_uri_get_fragment (const gchar *uri)
 {
     gchar *cur;
     gchar *frag_id = NULL;
@@ -387,7 +511,8 @@ get_doc_type (gchar *uri)
 	return YELP_DOC_TYPE_EXTERNAL;
 
     mime_type = gnome_vfs_get_mime_type (uri);
-    g_return_val_if_fail (mime_type != NULL, YELP_DOC_TYPE_ERROR);
+    if (mime_type == NULL)
+	return YELP_DOC_TYPE_ERROR;
 
     if (g_str_equal (mime_type, "text/xml"))
 	type = YELP_DOC_TYPE_DOCBOOK_XML;
@@ -402,10 +527,10 @@ get_doc_type (gchar *uri)
     return type;
 }
 
-static void
-doc_info_add_uri (YelpDocInfo *doc_info,
-		  gchar       *uri,
-		  YelpURIType  type)
+void
+yelp_doc_info_add_uri (YelpDocInfo *doc_info,
+		       const gchar *uri,
+		       YelpURIType  type)
 {
     DocInfoURI *info_uri;
 
@@ -561,11 +686,10 @@ convert_man_uri (gchar *uri)
     gchar *man_name = NULL;
     gchar *man_num  = NULL;
     gchar *man_dir  = NULL;
+    const gchar * const * langs = g_get_language_names ();
+    gint langs_i;
 
     static gchar **manpath = NULL;
-    static gchar *mandirs[] =
-	{"man1", "man2", "man3", "man4", "man5", "man6",
-	 "man7", "man8", "man9", "mann", NULL};
     gint i, j;
 
     GDir  *dir;
@@ -630,27 +754,34 @@ convert_man_uri (gchar *uri)
     g_free (pattern);
 
     for (i = 0; manpath[i]; i++) {
-	/* The man_dir/mandirs thing is probably cleverer than it should be. */
-	for (j = 0; man_dir ? (j < 1) : (mandirs[j] != NULL); j++) {
-	    dirname = g_build_filename (manpath[i],
-					man_dir ? man_dir : mandirs[j],
-					NULL);
-	    dir = g_dir_open (dirname, 0, NULL);
-	    if (dir) {
-		while ((filename = g_dir_read_name (dir))) {
-		    if (g_pattern_match_string (pspec, filename)) {
-			doc_uri = g_strconcat ("file://",
-					       dirname, "/",
-					       filename,
-					       NULL);
-			g_dir_close (dir);
-			g_free (dirname);
-			goto done;
+	for (langs_i = 0; langs[langs_i]; langs_i++) {
+	    for (j = 0; man_dir ? (j < 1) : (mandirs[j] != NULL); j++) {
+		if (g_str_equal (langs[langs_i], "C"))
+		    dirname = g_build_filename (manpath[i],
+						man_dir ? man_dir : mandirs[j],
+						NULL);
+		else
+		    dirname = g_build_filename (manpath[i],
+						langs[langs_i],
+						man_dir ? man_dir : mandirs[j],
+						NULL);
+		dir = g_dir_open (dirname, 0, NULL);
+		if (dir) {
+		    while ((filename = g_dir_read_name (dir))) {
+			if (g_pattern_match_string (pspec, filename)) {
+			    doc_uri = g_strconcat ("file://",
+						   dirname, "/",
+						   filename,
+						   NULL);
+			    g_dir_close (dir);
+			    g_free (dirname);
+			    goto done;
+			}
 		    }
+		    g_dir_close (dir);
 		}
-		g_dir_close (dir);
+		g_free (dirname);
 	    }
-	    g_free (dirname);
 	}
     }
 
